Package: VERSO
Version: 1.21.0
Date: 2025-09-23
Title: Viral Evolution ReconStructiOn (VERSO)
Authors@R: c(person("Daniele", "Ramazzotti", role=c("aut"),email="daniele.ramazzotti@unimib.it",
                comment = c(ORCID = "0000-0002-6087-2666")),
    person("Fabrizio","Angaroni",role=c("aut"),email="fabrizio.angaroni@unimib.it"),
    person("Davide","Maspero",role=c("cre","aut"),email="d.maspero@campus.unimib.it"),
    person("Alex","Graudenzi",role=c("aut"),email="alex.graudenzi@unimib.it"),
    person("Luca", "De Sano", role=c("aut"), email="luca.desano@gmail.com",
                comment = c(ORCID = "0000-0002-9618-3774")))
Depends: R (>= 4.1.0)
Imports: utils, data.tree, ape, parallel, Rfast, stats
Suggests: BiocGenerics, BiocStyle, testthat, knitr
Name: VERSO: an R package for the inference of viral evolution models
Description: 
    Mutations that rapidly accumulate in viral genomes during a pandemic can be used to track the evolution of the virus 
    and, accordingly, unravel the viral infection network. 
    To this extent, sequencing samples of the virus can be employed to estimate models from genomic epidemiology and may 
    serve, for instance, to estimate the proportion of undetected infected people by uncovering cryptic transmissions, as 
    well as to predict likely trends in the number of infected, hospitalized, dead and recovered people. 
    VERSO is an algorithmic framework that processes variants profiles from viral samples to produce phylogenetic 
    models of viral evolution. The approach solves a Boolean Matrix Factorization problem with phylogenetic constraints, 
    by maximizing a log-likelihood function. VERSO includes two separate and subsequent steps; in this package we provide 
    an R implementation of VERSO STEP 1. 
Encoding: UTF-8
License: file LICENSE
URL: https://github.com/BIMIB-DISCo/VERSO
BugReports: https://github.com/BIMIB-DISCo/VERSO
biocViews: BiomedicalInformatics, Sequencing, SomaticMutation
RoxygenNote: 7.3.2
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/VERSO
git_branch: devel
git_last_commit: 9a3fa06
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 23:34:29 UTC; biocbuild
Author: Daniele Ramazzotti [aut] (ORCID:
    <https://orcid.org/0000-0002-6087-2666>),
  Fabrizio Angaroni [aut],
  Davide Maspero [cre, aut],
  Alex Graudenzi [aut],
  Luca De Sano [aut] (ORCID: <https://orcid.org/0000-0002-9618-3774>)
Maintainer: Davide Maspero <d.maspero@campus.unimib.it>
