#' @name variants
#' @title Mutation data obtained by variant calling from raw data of a 
#' selected set of SARS-CoV-2 samples available from NCBI 
#' BioProject PRJNA610428.
#' @description The dataset includes variants for a selected set of 15 
#' SARS-CoV-2 samples obtained by variant calling from raw data 
#' available from NCBI BioProject PRJNA610428.
#' @docType data
#' @usage data(variants)
#' @format SARS-CoV-2 variants
#' @source NCBI BioProject PRJNA610428
#' @return SARS-CoV-2 variants
NULL

#' @name inference
#' @title Results obtained running VERSO on the provided input dataset.
#' @description Results obtained running VERSO on the provided input dataset.
#' @docType data
#' @usage data(inference)
#' @format results obtained running VERSO on the provided input dataset
#' @return results obtained running VERSO on the provided input dataset
NULL
